=== modified file 'NEWS'
--- NEWS	2005-10-17 11:56:54 +0000
+++ NEWS	2005-10-17 23:32:06 +0000
@@ -16,25 +16,25 @@
       Goffredo Baroncelli.
     
     * 'bzr check' now accepts -v for verbose reporting, and checks for
-      ghosts in the branch.
+      ghosts in the branch. (Robert Collins)
 
     * New command 're-sign' which will regenerate the gpg signature for 
-      a revision.
+      a revision. (Robert Collins)
 
     * If you set check_signatures=require for a path in 
       ~/.bazaar/branches.conf then bzr will invoke your
       gpg_signing_command (defaults to gpg) and record a digital signature
-      of your commit.
+      of your commit. (Robert Collins)
 
   TESTING:
 
     * The 'bzr selftest --pattern' option for has been removed, now 
       test specifiers on the command line can be simple strings, or 
-      regexps, or both.
+      regexps, or both. (Robert Collins)
 
     * Passing -v to selftest will now show the time each test took to 
       complete, which will aid in analysing performance regressions and
-      related questions.
+      related questions. (Robert Collins)
 
     * 'bzr selftest' runs all tests, even if one feels, unless '--one'
       is given.
@@ -69,6 +69,18 @@
       parameter, and if None is given will open the branch at basedir 
       implicitly. (Robert Collins)
 
+    * Branch.remove has been moved to WorkingTree, which has also gained
+      lock_read, lock_write and unlock methods for convenience. (Robert
+      Collins)
+
+    * Two decorators, needs_read_lock and needs_write_lock have been added
+      to the branch module. Use these to cause a function to run in a
+      read or write lock respectively. (Robert Collins)
+
+    * Branch.open_containing now returns a tuple (Branch, relative-path),
+      which allows direct access to the common case of 'get me this file
+      from its branch'. (Robert Collins)
+
 
 bzr 0.1.1 2005-10-12
 

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-10-16 22:31:25 +0000
+++ bzrlib/add.py	2005-10-17 23:13:00 +0000
@@ -66,7 +66,7 @@
     Returns the number of files added.
     """
     file_list = _prepare_file_list(file_list)
-    b = Branch.open_containing(file_list[0])
+    b = Branch.open_containing(file_list[0])[0]
     return smart_add_branch(b, file_list, recurse, reporter)
 
         

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2005-10-17 05:18:20 +0000
+++ bzrlib/annotate.py	2005-10-17 23:13:00 +0000
@@ -92,7 +92,7 @@
     from bzrlib.workingtree import WorkingTree
 
     enable_default_logging()
-    b = Branch.open_containing(sys.argv[1])
+    b = Branch.open_containing(sys.argv[1])[0]
     tree = WorkingTree(b.base, b)
     rp = tree.relpath(sys.argv[1])
     tree = b.revision_tree(b.last_revision())

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-17 11:56:54 +0000
+++ bzrlib/branch.py	2005-10-17 23:32:06 +0000
@@ -67,6 +67,28 @@
     raise NotImplementedError('find_branch() is not supported anymore, '
                               'please use one of the new branch constructors')
 
+
+def needs_read_lock(unbound):
+    """Decorate unbound to take out and release a read lock."""
+    def decorated(self, *args, **kwargs):
+        self.lock_read()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    return decorated
+
+
+def needs_write_lock(unbound):
+    """Decorate unbound to take out and release a write lock."""
+    def decorated(self, *args, **kwargs):
+        self.lock_write()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    return decorated
+
 ######################################################################
 # branch objects
 
@@ -103,11 +125,12 @@
 
         Basically we keep looking up until we find the control directory or
         run into the root.  If there isn't one, raises NotBranchError.
+        If there is one, it is returned, along with the unused portion of url.
         """
         t = get_transport(url)
         while True:
             try:
-                return _Branch(t)
+                return _Branch(t), t.relpath(url)
             except NotBranchError:
                 pass
             new_t = t.clone('..')
@@ -507,18 +530,15 @@
                 entry.parent_id = inv.root.file_id
         self._write_inventory(inv)
 
+    @needs_read_lock
     def read_working_inventory(self):
         """Read the working inventory."""
-        self.lock_read()
-        try:
-            # ElementTree does its own conversion from UTF-8, so open in
-            # binary.
-            f = self.controlfile('inventory', 'rb')
-            return bzrlib.xml5.serializer_v5.read_inventory(f)
-        finally:
-            self.unlock()
-            
+        # ElementTree does its own conversion from UTF-8, so open in
+        # binary.
+        f = self.controlfile('inventory', 'rb')
+        return bzrlib.xml5.serializer_v5.read_inventory(f)
 
+    @needs_write_lock
     def _write_inventory(self, inv):
         """Update the working inventory.
 
@@ -526,21 +546,18 @@
         will be committed to the next revision.
         """
         from cStringIO import StringIO
-        self.lock_write()
-        try:
-            sio = StringIO()
-            bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
-            sio.seek(0)
-            # Transport handles atomicity
-            self.put_controlfile('inventory', sio)
-        finally:
-            self.unlock()
+        sio = StringIO()
+        bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
+        sio.seek(0)
+        # Transport handles atomicity
+        self.put_controlfile('inventory', sio)
         
         mutter('wrote working inventory')
             
     inventory = property(read_working_inventory, _write_inventory, None,
                          """Inventory for the working copy.""")
 
+    @needs_write_lock
     def add(self, files, ids=None):
         """Make files versioned.
 
@@ -576,98 +593,45 @@
         else:
             assert(len(ids) == len(files))
 
-        self.lock_write()
-        try:
-            inv = self.read_working_inventory()
-            for f,file_id in zip(files, ids):
-                if is_control_file(f):
-                    raise BzrError("cannot add control file %s" % quotefn(f))
-
-                fp = splitpath(f)
-
-                if len(fp) == 0:
-                    raise BzrError("cannot add top-level %r" % f)
-
-                fullpath = os.path.normpath(self.abspath(f))
-
-                try:
-                    kind = file_kind(fullpath)
-                except OSError:
-                    # maybe something better?
-                    raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
-
-                if not InventoryEntry.versionable_kind(kind):
-                    raise BzrError('cannot add: not a versionable file ('
-                                   'i.e. regular file, symlink or directory): %s' % quotefn(f))
-
-                if file_id is None:
-                    file_id = gen_file_id(f)
-                inv.add_path(f, kind=kind, file_id=file_id)
-
-                mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-            
-
+        inv = self.read_working_inventory()
+        for f,file_id in zip(files, ids):
+            if is_control_file(f):
+                raise BzrError("cannot add control file %s" % quotefn(f))
+
+            fp = splitpath(f)
+
+            if len(fp) == 0:
+                raise BzrError("cannot add top-level %r" % f)
+
+            fullpath = os.path.normpath(self.abspath(f))
+
+            try:
+                kind = file_kind(fullpath)
+            except OSError:
+                # maybe something better?
+                raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
+
+            if not InventoryEntry.versionable_kind(kind):
+                raise BzrError('cannot add: not a versionable file ('
+                               'i.e. regular file, symlink or directory): %s' % quotefn(f))
+
+            if file_id is None:
+                file_id = gen_file_id(f)
+            inv.add_path(f, kind=kind, file_id=file_id)
+
+            mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
+
+        self._write_inventory(inv)
+
+    @needs_read_lock
     def print_file(self, file, revno):
         """Print `file` to stdout."""
-        self.lock_read()
-        try:
-            tree = self.revision_tree(self.get_rev_id(revno))
-            # use inventory as it was in that revision
-            file_id = tree.inventory.path2id(file)
-            if not file_id:
-                raise BzrError("%r is not present in revision %s" % (file, revno))
-            tree.print_file(file_id)
-        finally:
-            self.unlock()
-
-
-    def remove(self, files, verbose=False):
-        """Mark nominated files for removal from the inventory.
-
-        This does not remove their text.  This does not run on 
-
-        TODO: Refuse to remove modified files unless --force is given?
-
-        TODO: Do something useful with directories.
-
-        TODO: Should this remove the text or not?  Tough call; not
-        removing may be useful and the user can just use use rm, and
-        is the opposite of add.  Removing it is consistent with most
-        other tools.  Maybe an option.
-        """
-        ## TODO: Normalize names
-        ## TODO: Remove nested loops; better scalability
-        if isinstance(files, basestring):
-            files = [files]
-
-        self.lock_write()
-
-        try:
-            tree = self.working_tree()
-            inv = tree.inventory
-
-            # do this before any modifications
-            for f in files:
-                fid = inv.path2id(f)
-                if not fid:
-                    raise BzrError("cannot remove unversioned file %s" % quotefn(f))
-                mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
-                if verbose:
-                    # having remove it, it must be either ignored or unknown
-                    if tree.is_ignored(f):
-                        new_status = 'I'
-                    else:
-                        new_status = '?'
-                    show_status(new_status, inv[fid].kind, quotefn(f))
-                del inv[fid]
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
+        tree = self.revision_tree(self.get_rev_id(revno))
+        # use inventory as it was in that revision
+        file_id = tree.inventory.path2id(file)
+        if not file_id:
+            raise BzrError("%r is not present in revision %s" % (file, revno))
+        tree.print_file(file_id)
 
     # FIXME: this doesn't need to be a branch method
     def set_inventory(self, new_inventory_list):
@@ -700,23 +664,19 @@
         >>> b.add('foo')
         >>> list(b.unknowns())
         []
-        >>> b.remove('foo')
+        >>> WorkingTree(b.base, b).remove('foo')
         >>> list(b.unknowns())
         ['foo']
         """
         return self.working_tree().unknowns()
 
-
+    @needs_write_lock
     def append_revision(self, *revision_ids):
         for revision_id in revision_ids:
             mutter("add {%s} to revision-history" % revision_id)
-        self.lock_write()
-        try:
-            rev_history = self.revision_history()
-            rev_history.extend(revision_ids)
-            self.put_controlfile('revision-history', '\n'.join(rev_history))
-        finally:
-            self.unlock()
+        rev_history = self.revision_history()
+        rev_history.extend(revision_ids)
+        self.put_controlfile('revision-history', '\n'.join(rev_history))
 
     def has_revision(self, revision_id):
         """True if this branch has a copy of the revision.
@@ -726,19 +686,15 @@
         return (revision_id is None
                 or self.revision_store.has_id(revision_id))
 
+    @needs_read_lock
     def get_revision_xml_file(self, revision_id):
         """Return XML file object for revision object."""
         if not revision_id or not isinstance(revision_id, basestring):
             raise InvalidRevisionId(revision_id)
-
-        self.lock_read()
         try:
-            try:
-                return self.revision_store.get(revision_id)
-            except (IndexError, KeyError):
-                raise bzrlib.errors.NoSuchRevision(self, revision_id)
-        finally:
-            self.unlock()
+            return self.revision_store.get(revision_id)
+        except (IndexError, KeyError):
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
 
     #deprecated
     get_revision_xml = get_revision_xml_file
@@ -837,24 +793,21 @@
         else:
             return self.get_inventory(revision_id)
 
+    @needs_read_lock
     def revision_history(self):
         """Return sequence of revision hashes on to this branch."""
-        self.lock_read()
-        try:
-            transaction = self.get_transaction()
-            history = transaction.map.find_revision_history()
-            if history is not None:
-                mutter("cache hit for revision-history in %s", self)
-                return list(history)
-            history = [l.rstrip('\r\n') for l in
-                    self.controlfile('revision-history', 'r').readlines()]
-            transaction.map.add_revision_history(history)
-            # this call is disabled because revision_history is 
-            # not really an object yet, and the transaction is for objects.
-            # transaction.register_clean(history, precious=True)
+        transaction = self.get_transaction()
+        history = transaction.map.find_revision_history()
+        if history is not None:
+            mutter("cache hit for revision-history in %s", self)
             return list(history)
-        finally:
-            self.unlock()
+        history = [l.rstrip('\r\n') for l in
+                self.controlfile('revision-history', 'r').readlines()]
+        transaction.map.add_revision_history(history)
+        # this call is disabled because revision_history is 
+        # not really an object yet, and the transaction is for objects.
+        # transaction.register_clean(history, precious=True)
+        return list(history)
 
     def revno(self):
         """Return current revision number for this branch.
@@ -864,7 +817,6 @@
         """
         return len(self.revision_history())
 
-
     def last_revision(self):
         """Return last patch hash, or None if no history.
         """
@@ -874,7 +826,6 @@
         else:
             return None
 
-
     def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
         """Return a list of new revisions that would perfectly fit.
         
@@ -1009,56 +960,52 @@
         """
         return self.revision_tree(self.last_revision())
 
-
+    @needs_write_lock
     def rename_one(self, from_rel, to_rel):
         """Rename one file.
 
         This can change the directory or the filename or both.
         """
-        self.lock_write()
+        tree = self.working_tree()
+        inv = tree.inventory
+        if not tree.has_filename(from_rel):
+            raise BzrError("can't rename: old working file %r does not exist" % from_rel)
+        if tree.has_filename(to_rel):
+            raise BzrError("can't rename: new working file %r already exists" % to_rel)
+
+        file_id = inv.path2id(from_rel)
+        if file_id == None:
+            raise BzrError("can't rename: old name %r is not versioned" % from_rel)
+
+        if inv.path2id(to_rel):
+            raise BzrError("can't rename: new name %r is already versioned" % to_rel)
+
+        to_dir, to_tail = os.path.split(to_rel)
+        to_dir_id = inv.path2id(to_dir)
+        if to_dir_id == None and to_dir != '':
+            raise BzrError("can't determine destination directory id for %r" % to_dir)
+
+        mutter("rename_one:")
+        mutter("  file_id    {%s}" % file_id)
+        mutter("  from_rel   %r" % from_rel)
+        mutter("  to_rel     %r" % to_rel)
+        mutter("  to_dir     %r" % to_dir)
+        mutter("  to_dir_id  {%s}" % to_dir_id)
+
+        inv.rename(file_id, to_dir_id, to_tail)
+
+        from_abs = self.abspath(from_rel)
+        to_abs = self.abspath(to_rel)
         try:
-            tree = self.working_tree()
-            inv = tree.inventory
-            if not tree.has_filename(from_rel):
-                raise BzrError("can't rename: old working file %r does not exist" % from_rel)
-            if tree.has_filename(to_rel):
-                raise BzrError("can't rename: new working file %r already exists" % to_rel)
-
-            file_id = inv.path2id(from_rel)
-            if file_id == None:
-                raise BzrError("can't rename: old name %r is not versioned" % from_rel)
-
-            if inv.path2id(to_rel):
-                raise BzrError("can't rename: new name %r is already versioned" % to_rel)
-
-            to_dir, to_tail = os.path.split(to_rel)
-            to_dir_id = inv.path2id(to_dir)
-            if to_dir_id == None and to_dir != '':
-                raise BzrError("can't determine destination directory id for %r" % to_dir)
-
-            mutter("rename_one:")
-            mutter("  file_id    {%s}" % file_id)
-            mutter("  from_rel   %r" % from_rel)
-            mutter("  to_rel     %r" % to_rel)
-            mutter("  to_dir     %r" % to_dir)
-            mutter("  to_dir_id  {%s}" % to_dir_id)
-
-            inv.rename(file_id, to_dir_id, to_tail)
-
-            from_abs = self.abspath(from_rel)
-            to_abs = self.abspath(to_rel)
-            try:
-                rename(from_abs, to_abs)
-            except OSError, e:
-                raise BzrError("failed to rename %r to %r: %s"
-                        % (from_abs, to_abs, e[1]),
-                        ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-
+            rename(from_abs, to_abs)
+        except OSError, e:
+            raise BzrError("failed to rename %r to %r: %s"
+                    % (from_abs, to_abs, e[1]),
+                    ["rename rolled back"])
+
+        self._write_inventory(inv)
+
+    @needs_write_lock
     def move(self, from_paths, to_name):
         """Rename files.
 
@@ -1074,58 +1021,53 @@
         entry that is moved.
         """
         result = []
-        self.lock_write()
-        try:
-            ## TODO: Option to move IDs only
-            assert not isinstance(from_paths, basestring)
-            tree = self.working_tree()
-            inv = tree.inventory
-            to_abs = self.abspath(to_name)
-            if not isdir(to_abs):
-                raise BzrError("destination %r is not a directory" % to_abs)
-            if not tree.has_filename(to_name):
-                raise BzrError("destination %r not in working directory" % to_abs)
-            to_dir_id = inv.path2id(to_name)
-            if to_dir_id == None and to_name != '':
-                raise BzrError("destination %r is not a versioned directory" % to_name)
-            to_dir_ie = inv[to_dir_id]
-            if to_dir_ie.kind not in ('directory', 'root_directory'):
-                raise BzrError("destination %r is not a directory" % to_abs)
-
-            to_idpath = inv.get_idpath(to_dir_id)
-
-            for f in from_paths:
-                if not tree.has_filename(f):
-                    raise BzrError("%r does not exist in working tree" % f)
-                f_id = inv.path2id(f)
-                if f_id == None:
-                    raise BzrError("%r is not versioned" % f)
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                if tree.has_filename(dest_path):
-                    raise BzrError("destination %r already exists" % dest_path)
-                if f_id in to_idpath:
-                    raise BzrError("can't move %r to a subdirectory of itself" % f)
-
-            # OK, so there's a race here, it's possible that someone will
-            # create a file in this interval and then the rename might be
-            # left half-done.  But we should have caught most problems.
-
-            for f in from_paths:
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                result.append((f, dest_path))
-                inv.rename(inv.path2id(f), to_dir_id, name_tail)
-                try:
-                    rename(self.abspath(f), self.abspath(dest_path))
-                except OSError, e:
-                    raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
-                            ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
+        ## TODO: Option to move IDs only
+        assert not isinstance(from_paths, basestring)
+        tree = self.working_tree()
+        inv = tree.inventory
+        to_abs = self.abspath(to_name)
+        if not isdir(to_abs):
+            raise BzrError("destination %r is not a directory" % to_abs)
+        if not tree.has_filename(to_name):
+            raise BzrError("destination %r not in working directory" % to_abs)
+        to_dir_id = inv.path2id(to_name)
+        if to_dir_id == None and to_name != '':
+            raise BzrError("destination %r is not a versioned directory" % to_name)
+        to_dir_ie = inv[to_dir_id]
+        if to_dir_ie.kind not in ('directory', 'root_directory'):
+            raise BzrError("destination %r is not a directory" % to_abs)
+
+        to_idpath = inv.get_idpath(to_dir_id)
+
+        for f in from_paths:
+            if not tree.has_filename(f):
+                raise BzrError("%r does not exist in working tree" % f)
+            f_id = inv.path2id(f)
+            if f_id == None:
+                raise BzrError("%r is not versioned" % f)
+            name_tail = splitpath(f)[-1]
+            dest_path = appendpath(to_name, name_tail)
+            if tree.has_filename(dest_path):
+                raise BzrError("destination %r already exists" % dest_path)
+            if f_id in to_idpath:
+                raise BzrError("can't move %r to a subdirectory of itself" % f)
+
+        # OK, so there's a race here, it's possible that someone will
+        # create a file in this interval and then the rename might be
+        # left half-done.  But we should have caught most problems.
+
+        for f in from_paths:
+            name_tail = splitpath(f)[-1]
+            dest_path = appendpath(to_name, name_tail)
+            result.append((f, dest_path))
+            inv.rename(inv.path2id(f), to_dir_id, name_tail)
+            try:
+                rename(self.abspath(f), self.abspath(dest_path))
+            except OSError, e:
+                raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
+                        ["rename rolled back"])
+
+        self._write_inventory(inv)
         return result
 
 
@@ -1201,13 +1143,9 @@
         if updated:
             self.set_pending_merges(p)
 
+    @needs_write_lock
     def set_pending_merges(self, rev_list):
-        self.lock_write()
-        try:
-            self.put_controlfile('pending-merges', '\n'.join(rev_list))
-        finally:
-            self.unlock()
-
+        self.put_controlfile('pending-merges', '\n'.join(rev_list))
 
     def get_parent(self):
         """Return the parent location of the branch.
@@ -1226,20 +1164,16 @@
                     raise
         return None
 
-
+    @needs_write_lock
     def set_parent(self, url):
         # TODO: Maybe delete old location files?
         from bzrlib.atomicfile import AtomicFile
-        self.lock_write()
+        f = AtomicFile(self.controlfilename('parent'))
         try:
-            f = AtomicFile(self.controlfilename('parent'))
-            try:
-                f.write(url + '\n')
-                f.commit()
-            finally:
-                f.close()
+            f.write(url + '\n')
+            f.commit()
         finally:
-            self.unlock()
+            f.close()
 
     def check_revno(self, revno):
         """\
@@ -1258,13 +1192,13 @@
             raise InvalidRevisionNumber(revno)
         
     def sign_revision(self, revision_id, gpg_strategy):
-        self.lock_write()
-        try:
-            plaintext = Testament.from_revision(self, revision_id).as_short_text()
-            self.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), 
-                                    revision_id, "sig")
-        finally:
-            self.unlock()
+        plaintext = Testament.from_revision(self, revision_id).as_short_text()
+        self.store_revision_signature(gpg_strategy, plaintext, revision_id)
+
+    @needs_write_lock
+    def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
+        self.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), 
+                                revision_id, "sig")
 
 
 class ScratchBranch(_Branch):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-17 11:56:54 +0000
+++ bzrlib/builtins.py	2005-10-17 23:32:06 +0000
@@ -79,15 +79,17 @@
     
     def run(self, all=False, show_ids=False, file_list=None, revision=None):
         if file_list:
-            b = Branch.open_containing(file_list[0])
-            tree = WorkingTree(b.base, b)
-            file_list = [tree.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
+            b, relpath = Branch.open_containing(file_list[0])
+            if relpath == '' and len(file_list) == 1:
                 file_list = None
+            else:
+                # generate relative paths.
+                # note that if this is a remote branch, we would want
+                # relpath against the transport. RBC 20051018
+                tree = WorkingTree(b.base, b)
+                file_list = [tree.relpath(x) for x in file_list]
         else:
-            b = Branch.open_containing('.')
+            b = Branch.open_containing('.')[0]
             
         from bzrlib.status import show_status
         show_status(b, show_unchanged=all, show_ids=show_ids,
@@ -111,7 +113,7 @@
             raise BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id is None and revision is None:
             raise BzrCommandError('You must supply either --revision or a revision_id')
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         if revision_id is not None:
             sys.stdout.write(b.get_revision_xml_file(revision_id).read())
         elif revision is not None:
@@ -127,7 +129,7 @@
 
     This is equal to the number of revisions on this branch."""
     def run(self):
-        print Branch.open_containing('.').revno()
+        print Branch.open_containing('.')[0].revno()
 
 
 class cmd_revision_info(Command):
@@ -147,7 +149,7 @@
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
 
         for rev in revs:
             revinfo = rev.in_history(b)
@@ -205,7 +207,7 @@
         for d in dir_list:
             os.mkdir(d)
             if not b:
-                b = Branch.open_containing(d)
+                b = Branch.open_containing(d)[0]
             b.add([d])
             print 'added', d
 
@@ -216,8 +218,8 @@
     hidden = True
     
     def run(self, filename):
-        branch = Branch.open_containing(filename)
-        print WorkingTree(branch.base, branch).relpath(filename)
+        branch, relpath = Branch.open_containing(filename)
+        print relpath
 
 
 class cmd_inventory(Command):
@@ -225,7 +227,7 @@
     takes_options = ['revision', 'show-ids']
     
     def run(self, revision=None, show_ids=False):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         if revision is None:
             inv = b.read_working_inventory()
         else:
@@ -251,7 +253,7 @@
     """
     takes_args = ['source$', 'dest']
     def run(self, source_list, dest):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
 
         # TODO: glob expansion on windows?
         tree = WorkingTree(b.base, b)
@@ -275,7 +277,7 @@
     takes_args = ['from_name', 'to_name']
     
     def run(self, from_name, to_name):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         tree = WorkingTree(b.base, b)
         b.rename_one(tree.relpath(from_name), tree.relpath(to_name))
 
@@ -297,7 +299,7 @@
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
-        b = Branch.open_containing(names_list[0])
+        b = Branch.open_containing(names_list[0])[0]
         tree = WorkingTree(b.base, b)
         rel_names = [tree.relpath(x) for x in names_list]
         
@@ -338,7 +340,7 @@
         from shutil import rmtree
         import errno
         
-        br_to = Branch.open_containing('.')
+        br_to = Branch.open_containing('.')[0]
         stored_loc = br_to.get_parent()
         if location is None:
             if stored_loc is None:
@@ -410,7 +412,7 @@
         try:
             br_from.setup_caching(cache_root)
             if basis is not None:
-                basis_branch = Branch.open_containing(basis)
+                basis_branch = Branch.open_containing(basis)[0]
             else:
                 basis_branch = None
             if len(revision) == 1 and revision[0] is not None:
@@ -452,7 +454,7 @@
     takes_args = ['dir?']
 
     def run(self, dir='.'):
-        b = Branch.open_containing(dir)
+        b = Branch.open_containing(dir)[0]
         old_inv = b.basis_tree().inventory
         new_inv = b.read_working_inventory()
 
@@ -468,7 +470,7 @@
     
     def run(self, branch=None):
         import info
-        b = Branch.open_containing(branch)
+        b = Branch.open_containing(branch)[0]
         info.show_info(b)
 
 
@@ -483,9 +485,9 @@
     aliases = ['rm']
     
     def run(self, file_list, verbose=False):
-        b = Branch.open_containing(file_list[0])
+        b = Branch.open_containing(file_list[0])[0]
         tree = WorkingTree(b.base, b)
-        b.remove([tree.relpath(f) for f in file_list], verbose=verbose)
+        tree.remove([tree.relpath(f) for f in file_list], verbose=verbose)
 
 
 class cmd_file_id(Command):
@@ -498,9 +500,8 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = Branch.open_containing(filename)
-        tree = WorkingTree(b.base, b)
-        i = b.inventory.path2id(tree.relpath(filename))
+        b, relpath = Branch.open_containing(filename)
+        i = b.inventory.path2id(relpath)
         if i == None:
             raise BzrError("%r is not a versioned file" % filename)
         else:
@@ -515,10 +516,9 @@
     hidden = True
     takes_args = ['filename']
     def run(self, filename):
-        b = Branch.open_containing(filename)
+        b, relpath = Branch.open_containing(filename)
         inv = b.inventory
-        tree = WorkingTree(b.base, b)
-        fid = inv.path2id(tree.relpath(filename))
+        fid = inv.path2id(relpath)
         if fid == None:
             raise BzrError("%r is not a versioned file" % filename)
         for fip in inv.get_idpath(fid):
@@ -529,7 +529,7 @@
     """Display list of revision ids on this branch."""
     hidden = True
     def run(self):
-        for patchid in Branch.open_containing('.').revision_history():
+        for patchid in Branch.open_containing('.')[0].revision_history():
             print patchid
 
 
@@ -545,7 +545,7 @@
 class cmd_directories(Command):
     """Display list of versioned directories in this branch."""
     def run(self):
-        for name, ie in Branch.open_containing('.').read_working_inventory().directories():
+        for name, ie in Branch.open_containing('.')[0].read_working_inventory().directories():
             if name == '':
                 print '.'
             else:
@@ -602,14 +602,14 @@
         from bzrlib.diff import show_diff
 
         if file_list:
-            b = Branch.open_containing(file_list[0])
+            b = Branch.open_containing(file_list[0])[0]
             tree = WorkingTree(b.base, b)
             file_list = [tree.relpath(f) for f in file_list]
             if file_list == ['']:
                 # just pointing to top-of-tree
                 file_list = None
         else:
-            b = Branch.open_containing('.')
+            b = Branch.open_containing('.')[0]
 
         if revision is not None:
             if len(revision) == 1:
@@ -638,7 +638,7 @@
     # level of effort but possibly much less IO.  (Or possibly not,
     # if the directories are very large...)
     def run(self, show_ids=False):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         old = b.basis_tree()
         new = b.working_tree()
         for path, ie in old.inventory.iter_entries():
@@ -655,7 +655,7 @@
     def run(self):
         from bzrlib.delta import compare_trees
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         td = compare_trees(b.basis_tree(), b.working_tree())
 
         for path, id, kind, text_modified, meta_modified in td.modified:
@@ -667,7 +667,7 @@
     """List files added in working tree."""
     hidden = True
     def run(self):
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         wt = b.working_tree()
         basis_inv = b.basis_tree().inventory
         inv = wt.inventory
@@ -689,7 +689,7 @@
     takes_args = ['filename?']
     def run(self, filename=None):
         """Print the branch root."""
-        b = Branch.open_containing(filename)
+        b = Branch.open_containing(filename)[0]
         print b.base
 
 
@@ -732,15 +732,13 @@
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
-            b = Branch.open_containing(filename)
-            tree = WorkingTree(b.base, b)
-            fp = tree.relpath(filename)
-            if fp:
+            b, fp = Branch.open_containing(filename)
+            if fp != '':
                 file_id = b.read_working_inventory().path2id(fp)
             else:
                 file_id = None  # points to branch root
         else:
-            b = Branch.open_containing('.')
+            b, relpath = Branch.open_containing('.')
             file_id = None
 
         if revision is None:
@@ -793,10 +791,9 @@
     hidden = True
     takes_args = ["filename"]
     def run(self, filename):
-        b = Branch.open_containing(filename)
+        b, relpath = Branch.open_containing(filename)[0]
         inv = b.read_working_inventory()
-        tree = WorkingTree(b.base, b)
-        file_id = inv.path2id(tree.relpath(filename))
+        file_id = inv.path2id(relpath)
         for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
             print "%6d %s" % (revno, what)
 
@@ -807,7 +804,7 @@
     # TODO: Take a revision or remote path and list that tree instead.
     hidden = True
     def run(self, revision=None, verbose=False):
-        b = Branch.open_containing('.')
+        b, relpath = Branch.open_containing('.')[0]
         if revision == None:
             tree = b.working_tree()
         else:
@@ -825,7 +822,7 @@
     """List unknown files."""
     def run(self):
         from bzrlib.osutils import quotefn
-        for f in Branch.open_containing('.').unknowns():
+        for f in Branch.open_containing('.')[0].unknowns():
             print quotefn(f)
 
 
@@ -855,7 +852,7 @@
         from bzrlib.atomicfile import AtomicFile
         import os.path
 
-        b = Branch.open_containing('.')
+        b, relpath = Branch.open_containing('.')
         ifn = b.abspath('.bzrignore')
 
         if os.path.exists(ifn):
@@ -895,7 +892,7 @@
 
     See also: bzr ignore"""
     def run(self):
-        tree = Branch.open_containing('.').working_tree()
+        tree = Branch.open_containing('.')[0].working_tree()
         for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
@@ -919,7 +916,7 @@
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print Branch.open_containing('.').get_rev_id(revno)
+        print Branch.open_containing('.')[0].get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -938,7 +935,7 @@
     takes_options = ['revision', 'format', 'root']
     def run(self, dest, revision=None, format=None, root=None):
         import os.path
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         if revision is None:
             rev_id = b.last_revision()
         else:
@@ -977,9 +974,8 @@
             raise BzrCommandError("bzr cat requires a revision number")
         elif len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = Branch.open_containing('.')
-        tree = WorkingTree(b.base, b)
-        b.print_file(tree.relpath(filename), revision[0].in_history(b).revno)
+        b, relpath = Branch.open_containing(filename)
+        b.print_file(relpath, revision[0].in_history(b).revno)
 
 
 class cmd_local_time_offset(Command):
@@ -1027,7 +1023,7 @@
         from bzrlib.status import show_status
         from cStringIO import StringIO
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         tree = WorkingTree(b.base, b)
         if selected_list:
             selected_list = [tree.relpath(s) for s in selected_list]
@@ -1075,7 +1071,7 @@
 
     def run(self, dir='.', verbose=False):
         from bzrlib.check import check
-        check(Branch.open_containing(dir), verbose)
+        check(Branch.open_containing(dir)[0], verbose)
 
 
 class cmd_scan_cache(Command):
@@ -1120,7 +1116,7 @@
     
     def run(self, email=False):
         try:
-            b = bzrlib.branch.Branch.open_containing('.')
+            b = bzrlib.branch.Branch.open_containing('.')[0]
             config = bzrlib.config.BranchConfig(b)
         except NotBranchError:
             config = bzrlib.config.GlobalConfig()
@@ -1212,8 +1208,8 @@
     def run(self, branch, other):
         from bzrlib.revision import common_ancestor, MultipleRevisionSources
         
-        branch1 = Branch.open_containing(branch)
-        branch2 = Branch.open_containing(other)
+        branch1 = Branch.open_containing(branch)[0]
+        branch2 = Branch.open_containing(other)[0]
 
         history_1 = branch1.revision_history()
         history_2 = branch2.revision_history()
@@ -1277,7 +1273,7 @@
         if merge_type is None:
             merge_type = ApplyMerge3
         if branch is None:
-            branch = Branch.open_containing('.').get_parent()
+            branch = Branch.open_containing('.')[0].get_parent()
             if branch is None:
                 raise BzrCommandError("No merge location known or specified.")
             else:
@@ -1334,7 +1330,7 @@
         elif len(revision) != 1:
             raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
         else:
-            b = Branch.open_containing('.')
+            b = Branch.open_containing('.')[0]
             revno = revision[0].in_history(b).revno
         merge(('.', revno), parse_spec('.'),
               check_clean=False,
@@ -1342,7 +1338,7 @@
               backup_files=not no_backup,
               file_list=file_list)
         if not file_list:
-            Branch.open_containing('.').set_pending_merges([])
+            Branch.open_containing('.')[0].set_pending_merges([])
 
 
 class cmd_assert_fail(Command):
@@ -1414,7 +1410,7 @@
         if verbose and quiet:
             raise BzrCommandError('Cannot pass both quiet and verbose')
 
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         parent = b.get_parent()
         if remote is None:
             if parent is None:
@@ -1427,7 +1423,7 @@
             # We only update parent if it did not exist, missing
             # should not change the parent
             b.set_parent(remote)
-        br_remote = Branch.open_containing(remote)
+        br_remote = Branch.open_containing(remote)[0]
         return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
 
 
@@ -1456,7 +1452,7 @@
     takes_args = ['branch?']
     def run(self, branch='.', revision=None, long=False):
         from bzrlib.testament import Testament
-        b = Branch.open_containing(branch)
+        b = Branch.open_containing(branch)[0]
         b.lock_read()
         try:
             if revision is None:
@@ -1493,13 +1489,12 @@
 
     def run(self, filename, all=False, long=False):
         from bzrlib.annotate import annotate_file
-        b = Branch.open_containing(filename)
+        b, relpath = Branch.open_containing(filename)
         b.lock_read()
         try:
             tree = WorkingTree(b.base, b)
-            rp = tree.relpath(filename)
             tree = b.revision_tree(b.last_revision())
-            file_id = tree.inventory.path2id(rp)
+            file_id = tree.inventory.path2id(relpath)
             file_version = tree.inventory[file_id].revision
             annotate_file(b, file_version, file_id, long, all, sys.stdout)
         finally:
@@ -1521,7 +1516,7 @@
             raise BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id is None and revision is None:
             raise BzrCommandError('You must supply either --revision or a revision_id')
-        b = Branch.open_containing('.')
+        b = Branch.open_containing('.')[0]
         gpg_strategy = gpg.GPGStrategy(config.BranchConfig(b))
         if revision_id is not None:
             b.sign_revision(revision_id, gpg_strategy)

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-17 11:31:05 +0000
+++ bzrlib/commit.py	2005-10-17 21:57:32 +0000
@@ -79,6 +79,7 @@
                            )
 import bzrlib.gpg as gpg
 from bzrlib.revision import Revision
+from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
 from bzrlib.inventory import Inventory, ROOT_ID
@@ -321,9 +322,11 @@
         rev_tmp = StringIO()
         serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
+        if self.config.signature_needed():
+            plaintext = Testament(self.rev, self.new_inv).as_short_text()
+            self.branch.store_revision_signature(gpg.GPGStrategy(self.config),
+                                                 plaintext, self.rev_id)
         self.branch.revision_store.add(rev_tmp, self.rev_id)
-        if self.config.signature_needed():
-            self.branch.sign_revision(self.rev_id, gpg.GPGStrategy(self.config))
         mutter('new revision_id is {%s}', self.rev_id)
 
     def _remove_deleted(self):

=== modified file 'bzrlib/conflicts.py'
--- bzrlib/conflicts.py	2005-10-13 07:06:01 +0000
+++ bzrlib/conflicts.py	2005-10-17 23:13:00 +0000
@@ -37,7 +37,7 @@
     files.)
     """
     def run(self):
-        for path in Branch.open_containing('.').working_tree().iter_conflicts():
+        for path in Branch.open_containing('.')[0].working_tree().iter_conflicts():
             print path
 
 class cmd_resolve(bzrlib.commands.Command):
@@ -50,7 +50,7 @@
             if not all:
                 raise BzrCommandError(
                     "command 'resolve' needs one or more FILE, or --all")
-            tree = Branch.open_containing('.').working_tree()
+            tree = Branch.open_containing('.')[0].working_tree()
             file_list = list(tree.abspath(f) for f in tree.iter_conflicts())
         else:
             if all:

=== modified file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	2005-10-17 10:50:39 +0000
+++ bzrlib/gpg.py	2005-10-17 21:57:32 +0000
@@ -22,7 +22,18 @@
 
 import bzrlib.errors as errors
 
+class DisabledGPGStrategy(object):
+    """A GPG Strategy that makes everything fail."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        raise errors.SigningFailed('Signing is disabled.')
+
+
 class LoopbackGPGStrategy(object):
+    """A GPG Strategy that acts like 'cat' - data is just passed through."""
 
     def __init__(self, ignored):
         """Real strategies take a configuration."""

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-10-17 05:18:20 +0000
+++ bzrlib/merge.py	2005-10-17 23:13:00 +0000
@@ -199,7 +199,7 @@
             
 def get_tree(treespec, local_branch=None):
     location, revno = treespec
-    branch = Branch.open_containing(location)
+    branch = Branch.open_containing(location)[0]
     if revno is None:
         revision = None
     elif revno == -1:
@@ -264,7 +264,7 @@
     try:
         if this_dir is None:
             this_dir = '.'
-        this_branch = Branch.open_containing(this_dir)
+        this_branch = Branch.open_containing(this_dir)[0]
         this_rev_id = this_branch.last_revision()
         if this_rev_id is None:
             raise BzrCommandError("This branch has no commits")

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-10-16 14:51:21 +0000
+++ bzrlib/revisionspec.py	2005-10-17 23:13:00 +0000
@@ -313,7 +313,7 @@
     def _match_on(self, branch, revs):
         from branch import Branch
         from revision import common_ancestor, MultipleRevisionSources
-        other_branch = Branch.open_containing(self.spec)
+        other_branch = Branch.open_containing(self.spec)[0]
         revision_a = branch.last_revision()
         revision_b = other_branch.last_revision()
         for r, b in ((revision_a, branch), (revision_b, other_branch)):
@@ -339,7 +339,7 @@
     def _match_on(self, branch, revs):
         from branch import Branch
         from fetch import greedy_fetch
-        other_branch = Branch.open_containing(self.spec)
+        other_branch = Branch.open_containing(self.spec)[0]
         revision_b = other_branch.last_revision()
         if revision_b is None:
             raise NoCommits(other_branch)

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-10-17 11:56:54 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-17 23:13:00 +0000
@@ -821,4 +821,15 @@
         branch = Branch.open('to')
         self.assertEqual(1, len(branch.revision_history()))
 
+    def test_log(self):
+        self.build_tree(['branch/', 'branch/file'])
+        branch = Branch.initialize('branch')
+        branch.add(['file'])
+        branch.commit('add file', rev_id='A')
+        url = self.get_remote_url('branch/file')
+        output = self.capture('log %s' % url)
+        self.assertEqual(7, len(output.split('\n')))
+        
+
+
 

=== modified file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	2005-10-17 11:31:05 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-17 21:57:32 +0000
@@ -21,7 +21,7 @@
 from bzrlib.branch import Branch
 from bzrlib.commit import Commit
 from bzrlib.config import BranchConfig
-from bzrlib.errors import PointlessCommit, BzrError
+from bzrlib.errors import PointlessCommit, BzrError, SigningFailed
 
 
 # TODO: Test commit with some added, and added-but-missing files
@@ -274,3 +274,28 @@
                              branch.revision_store.get('B', 'sig').read())
         finally:
             bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_failed_signature(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
+            config = MustSignConfig(branch)
+            self.assertRaises(SigningFailed,
+                              commit.Commit(config=config).commit,
+                              branch, "base",
+                              allow_pointless=True,
+                              rev_id='B')
+            branch = Branch.open('.')
+            self.assertEqual(branch.revision_history(), ['A'])
+            self.failIf(branch.revision_store.has_id('B'))
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-10-17 11:56:54 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-17 23:13:00 +0000
@@ -15,15 +15,17 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-from bzrlib.branch import Branch
+
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.clone import copy_branch
 from bzrlib.commit import commit
 import bzrlib.errors as errors
 from bzrlib.errors import NoSuchRevision, UnlistableBranch, NotBranchError
-from bzrlib.selftest import TestCaseInTempDir
+import bzrlib.gpg
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.trace import mutter
 import bzrlib.transactions as transactions
-from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
 
 # TODO: Make a branch using basis branch, and check that it 
 # doesn't request any files that could have been avoided, by 
@@ -143,7 +145,6 @@
         self.assertEquals(b.pending_merges(), [])
 
     def test_sign_existing_revision(self):
-        import bzrlib.gpg
         branch = Branch.initialize('.')
         branch.commit("base", allow_pointless=True, rev_id='A')
         from bzrlib.testament import Testament
@@ -151,6 +152,12 @@
         self.assertEqual(Testament.from_revision(branch, 'A').as_short_text(),
                          branch.revision_store.get('A', 'sig').read())
 
+    def test_store_signature(self):
+        branch = Branch.initialize('.')
+        branch.store_revision_signature(bzrlib.gpg.LoopbackGPGStrategy(None),
+                                        'FOO', 'A')
+        self.assertEqual('FOO', branch.revision_store.get('A', 'sig').read())
+
 
 class TestRemote(TestCaseWithWebserver):
 
@@ -160,8 +167,10 @@
         self.assertRaises(NotBranchError, Branch.open_containing,
                           self.get_remote_url('g/p/q'))
         b = Branch.initialize('.')
-        Branch.open_containing(self.get_remote_url(''))
-        Branch.open_containing(self.get_remote_url('g/p/q'))
+        branch, relpath = Branch.open_containing(self.get_remote_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = Branch.open_containing(self.get_remote_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)
         
 # TODO: rewrite this as a regular unittest, without relying on the displayed output        
 #         >>> from bzrlib.commit import commit
@@ -191,6 +200,60 @@
         self.calls = []
 
 
+class TestDecorator(object):
+
+    def __init__(self):
+        self._calls = []
+
+    def lock_read(self):
+        self._calls.append('lr')
+
+    def lock_write(self):
+        self._calls.append('lw')
+
+    def unlock(self):
+        self._calls.append('ul')
+
+    @needs_read_lock
+    def do_with_read(self):
+        return 1
+
+    @needs_read_lock
+    def except_with_read(self):
+        raise RuntimeError
+
+    @needs_write_lock
+    def do_with_write(self):
+        return 2
+
+    @needs_write_lock
+    def except_with_write(self):
+        raise RuntimeError
+
+
+class TestDecorators(TestCase):
+
+    def test_needs_read_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(1, branch.do_with_read())
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_excepts_in_read_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_read)
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_needs_write_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(2, branch.do_with_write())
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+    def test_excepts_in_write_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_write)
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+
 class TestBranchTransaction(TestCaseInTempDir):
 
     def setUp(self):

=== modified file 'bzrlib/selftest/testgpg.py'
--- bzrlib/selftest/testgpg.py	2005-10-17 08:15:09 +0000
+++ bzrlib/selftest/testgpg.py	2005-10-17 21:57:32 +0000
@@ -50,3 +50,10 @@
         my_gpg._command_line = lambda:["cat", "-"]
         self.assertEqual("some content\nwith newlines\n",
                          my_gpg.sign("some content\nwith newlines\n"))
+
+
+class TestDisabled(TestCase):
+
+    def test_sign(self):
+        self.assertRaises(errors.SigningFailed,
+                          gpg.DisabledGPGStrategy(None).sign, 'content')

=== modified file 'bzrlib/selftest/testtestament.py'
--- bzrlib/selftest/testtestament.py	2005-10-17 06:57:11 +0000
+++ bzrlib/selftest/testtestament.py	2005-10-17 21:57:32 +0000
@@ -29,7 +29,9 @@
 from bzrlib.trace import mutter
 from bzrlib.osutils import has_symlinks
 
+
 class TestamentTests(TestCaseInTempDir):
+
     def setUp(self):
         super(TestamentTests, self).setUp()
         b = self.b = Branch.initialize('.')
@@ -114,6 +116,14 @@
                       revprops=props)
         t = Testament.from_revision(self.b, 'test@user-3')
         self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+
+    def test___init__(self):
+        revision = self.b.get_revision('test@user-2')
+        inventory = self.b.get_inventory('test@user-2')
+        testament_1 = Testament(revision, inventory).as_short_text()
+        testament_2 = Testament.from_revision(self.b, 
+                                              'test@user-2').as_short_text()
+        self.assertEqual(testament_1, testament_2)
                     
 
 REV_1_TESTAMENT = """\

=== modified file 'bzrlib/selftest/testworkingtree.py'
--- bzrlib/selftest/testworkingtree.py	2005-10-16 22:09:09 +0000
+++ bzrlib/selftest/testworkingtree.py	2005-10-17 23:32:06 +0000
@@ -79,3 +79,17 @@
         tree = WorkingTree(branch.base)
         self.assertEqual('child',
                          tree.relpath(os.path.join(os.getcwd(), 'child')))
+
+    def test_lock_locks_branch(self):
+        branch = Branch.initialize('.')
+        tree = WorkingTree(branch.base)
+        tree.lock_read()
+        self.assertEqual(1, tree.branch._lock_count)
+        self.assertEqual('r', tree.branch._lock_mode)
+        tree.unlock()
+        self.assertEqual(None, tree.branch._lock_count)
+        tree.lock_write()
+        self.assertEqual(1, tree.branch._lock_count)
+        self.assertEqual('w', tree.branch._lock_mode)
+        tree.unlock()
+        self.assertEqual(None, tree.branch._lock_count)

=== modified file 'bzrlib/testament.py'
--- bzrlib/testament.py	2005-10-17 06:57:11 +0000
+++ bzrlib/testament.py	2005-10-17 21:57:32 +0000
@@ -90,19 +90,22 @@
     @classmethod
     def from_revision(cls, branch, revision_id):
         """Produce a new testament from a historical revision"""
-        t = cls()
         rev = branch.get_revision(revision_id)
-        t.revision_id = str(revision_id)
-        t.committer = rev.committer
-        t.timezone = rev.timezone or 0
-        t.timestamp = rev.timestamp
-        t.message = rev.message
-        t.parent_ids = rev.parent_ids[:]
-        t.inventory = branch.get_inventory(revision_id)
-        t.revprops = copy(rev.properties)
-        assert not contains_whitespace(t.revision_id)
-        assert not contains_linebreaks(t.committer)
-        return t
+        inventory = branch.get_inventory(revision_id)
+        return cls(rev, inventory)
+
+    def __init__(self, rev, inventory):
+        """Create a new testament for rev using inventory."""
+        self.revision_id = str(rev.revision_id)
+        self.committer = rev.committer
+        self.timezone = rev.timezone or 0
+        self.timestamp = rev.timestamp
+        self.message = rev.message
+        self.parent_ids = rev.parent_ids[:]
+        self.inventory = inventory
+        self.revprops = copy(rev.properties)
+        assert not contains_whitespace(self.revision_id)
+        assert not contains_linebreaks(self.committer)
 
     def as_text_lines(self):
         """Yield text form as a sequence of lines.
@@ -132,7 +135,7 @@
         r.extend(self._revprops_to_lines())
         if __debug__:
             for l in r:
-                assert isinstance(l, str), \
+                assert isinstance(l, basestring), \
                     '%r of type %s is not a plain string' % (l, type(l))
         return r
 

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-10-17 00:00:33 +0000
+++ bzrlib/transport/local.py	2005-10-17 23:13:00 +0000
@@ -68,6 +68,8 @@
         """Return the local path portion from a given absolute path.
         """
         from bzrlib.osutils import relpath
+        if abspath is None:
+            abspath = '.'
         return relpath(self.base, abspath)
 
     def has(self, relpath):

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-17 05:18:20 +0000
+++ bzrlib/workingtree.py	2005-10-17 23:32:06 +0000
@@ -24,7 +24,7 @@
 import stat
 import fnmatch
  
-from bzrlib.branch import Branch
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock, quotefn
 import bzrlib.tree
 from bzrlib.osutils import appendpath, file_kind, isdir, splitpath, relpath
 from bzrlib.errors import BzrCheckError
@@ -94,6 +94,7 @@
     It is possible for a `WorkingTree` to have a filename which is
     not listed in the Inventory and vice versa.
     """
+
     def __init__(self, basedir, branch=None):
         """Construct a WorkingTree for basedir.
 
@@ -398,6 +399,65 @@
     def kind(self, file_id):
         return file_kind(self.id2abspath(file_id))
 
+    def lock_read(self):
+        """See Branch.lock_read, and WorkingTree.unlock."""
+        return self.branch.lock_read()
+
+    def lock_write(self):
+        """See Branch.lock_write, and WorkingTree.unlock."""
+        return self.branch.lock_write()
+
+    @needs_write_lock
+    def remove(self, files, verbose=False):
+        """Remove nominated files from the working inventory..
+
+        This does not remove their text.  This does not run on XXX on what? RBC
+
+        TODO: Refuse to remove modified files unless --force is given?
+
+        TODO: Do something useful with directories.
+
+        TODO: Should this remove the text or not?  Tough call; not
+        removing may be useful and the user can just use use rm, and
+        is the opposite of add.  Removing it is consistent with most
+        other tools.  Maybe an option.
+        """
+        ## TODO: Normalize names
+        ## TODO: Remove nested loops; better scalability
+        if isinstance(files, basestring):
+            files = [files]
+
+        inv = self.inventory
+
+        # do this before any modifications
+        for f in files:
+            fid = inv.path2id(f)
+            if not fid:
+                raise BzrError("cannot remove unversioned file %s" % quotefn(f))
+            mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
+            if verbose:
+                # having remove it, it must be either ignored or unknown
+                if self.is_ignored(f):
+                    new_status = 'I'
+                else:
+                    new_status = '?'
+                show_status(new_status, inv[fid].kind, quotefn(f))
+            del inv[fid]
+
+        self.branch._write_inventory(inv)
+
+    def unlock(self):
+        """See Branch.unlock.
+        
+        WorkingTree locking just uses the Branch locking facilities.
+        This is current because all working trees have an embedded branch
+        within them. IF in the future, we were to make branch data shareable
+        between multiple working trees, i.e. via shared storage, then we 
+        would probably want to lock both the local tree, and the branch.
+        """
+        return self.branch.unlock()
+
+
 CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
 def get_conflicted_stem(path):
     for suffix in CONFLICT_SUFFIXES:

=== modified file 'setup.py' (properties changed: -x to +x)
